function [Fvec,Xvec,infodat] = hinfsopt(Avec,Bvec,Dvec,Mvec,Jvec,P,nu,opt1,opt2);

%HINFSOPT Finds a solution for the suboptimal Hinf control problem.
%
%	Usage:
%		[Fvec,Xvec] = hinfsopt(Avec,Bvec,Dvec,Mvec,Jvec,P,NU,OPT1,OPT2);
%
%	Fvec contains the Hinf controller and Xvec contains the solution of the
%	Hinf CARE.
%
%	Avec,...,Jvec are the vectorized system matrices, P is the transition
%	probability matrix and NU is the upper bound for the Hinf norm.
%
%	Vector OPT1 contains options for the initial convex programming problem
%	(type HELP MINCX for more information)
%
%	Vector OPT2 contains options for the recursive part of the algorithm:
%
%		OPT2 = [TOL MAXITER VERB]
%
%	where TOL is the relative tolerance for convergence (default 1e-3),
%	MAXITER is the maximum number of iterations (default 200) and VERB,
%	which stands for "verbose" controls the output of information of
%	the initial problem construction and iteration phases (0=off, 1=on).
%	Default value is 0 (no additional information).
%	
%
%	See also STACK, UNSTACK, MAXCARE.
%	(use the command TYPE HINFSOPT for additional information)

%-------ADDITIONAL INFORMATION--------------------------------------------------
%
%	Batch syntax for HINFSOPT:
%
%		[Fvec,Xvec,INFODAT]=hinfsopt(Avec,...,OPT2);	
%
%	Batch syntax is used for invoking the function inside more complex
%	algorithms. The basic difference between batch syntax and normal
%	syntax is that when used with the latter, the function will stop with
%	error when the problem has no solution. In batch syntax only a vector
%	with information about the error condition is passed to the invoking
%	routine.
%
%	INFODAT = [ NITER COND1 COND2 ] contains the following information:
%
%		NITER.....Number of iterations. A minus signal indicates that
%		          there was no convergence (e.g. -200 means that the
%		          algorithm reached MAXITER=200 without convergence).
%		COND1.....COND1=0 means that condition (i) checking is OK.
%		COND2.....COND2=0 means that condition (ii) checking is OK.
%
%-------------------------------------------------------------------------------

%	RP Marques
%	Version 1.0	(05/03/97)

% Argument checking & preliminaries
if     nargin < 8,
	opt1=[0 0 0 0 1]; opt2=[];
elseif nargin <9
	opt2=[];
end;

if nargout > 2
	flag = 1;	% batch syntax
	infodat = [0 0 0];
else
	flag = 0;	% normal syntax
end;

if opt2==[], TOL=1e-3; MAXITER=200; VERB=0;
else,        TOL=opt2(1); MAXITER=opt2(2); VERB=opt2(3);
end;

if TOL==0,     TOL=1e-3; end;
if MAXITER==0, MAXITER=200; end;

N=Avec(2);

% ---------------------
% 1) Initial conditions
if VERB
	fprintf('\n\n-------------------------------\n');
	fprintf(    'Suboptimal Hinf control Problem\n');
	fprintf(    'Upper bound (nu): %d\n',nu);
	fprintf(    '-------------------------------\n\n');
	fprintf(    '1. Initial condition\n\n');
end;

% 1.1 Qvec
str=[];
for i=1:N
	istr=int2str(i); aux=[',Q',istr]; str=[str,aux];
	eval(['M',istr,'=unstack(Mvec,',istr,');']);
	eval(['Q',istr,'=M',istr,'''*M',istr,';']);	
end;
str(1)=[]; eval(['Qvec=stack(',str,');']);

% 1.2 Ivec
dimI=size(unstack(Bvec,1),2); 
str=[];
for i=1:N
	istr=int2str(i); aux=[',I',istr]; str=[str,aux];
	eval(['I',istr,'=eye(dimI);']);
end;
str(1)=[]; eval(['Ivec=stack(',str,');']);

% 1.3 Ovec
dimA=size(unstack(Avec,1),1);
str=[];
for i=1:N
	istr=int2str(i); aux=[',O',istr]; str=[str,aux];
	eval(['O',istr,'=zeros(dimA,dimI);']);
end;
str(1)=[]; eval(['Ovec=stack(',str,');']);

% 1.4 Initial solution
Xold = maxcare(Avec,Bvec,Qvec,Ovec,Ivec,P,opt1,0);


% ------------------
% 2) Iteration phase
if VERB
	fprintf('\n2. Iteration phase\n\n');
	fprintf(  '--it.#----Residue-----------TOLratio-----\n');
end;

r  = inf; % Initializing error
it = 1;   % Iterations

% Problem dimensions
dimU = size(unstack(Bvec,1),2);
dimW = size(unstack(Dvec,1),2);
dimX = size(unstack(Avec,1),2);

% Auxiliary matrices
Idbloc = [[eye(dimU) zeros(dimU,dimW)];[zeros(dimW,dimU) -eye(dimW)]];
IU=eye(dimU); 
IW=eye(dimW);
IX=eye(dimX);

while r > TOL & it <= MAXITER

	% 2.1 Expected value
	Evec=eop(Xold,P);

	% 2.2 New value for X
	str=[];
	for i=1:N
		% 2.2.1 Auxiliary values
		istr=int2str(i); aux=[',X',istr]; str=[str,aux];
		BnuD   = [unstack(Bvec,i) unstack(Dvec,i)/nu];
		Ai = unstack(Avec,i); Ei = unstack(Evec,i); Qi = unstack(Qvec,i);
		% 2.2.2 Operator M
		Xi = Qi+Ai'*Ei*Ai - Ai'*Ei*BnuD*inv(Idbloc + BnuD'*Ei*BnuD)*BnuD'*Ei*Ai;
		eval(['X',istr,'=Xi;']);
	end;
	str(1)=[]; eval(['Xnew=stack(',str,');']);		
	

	% 2.3 Residue
	r = res(Xnew,Xold);

	% 2.4 Output
	if VERB
		fprintf('%6.0f\t  %d\t    %d\n',it,r,r/TOL);
	end;
	
	% 2.5 Updating
	Xold=Xnew; it=it+1;
end;

if     it <= MAXITER & flag == 1
	infodat(1) = it;
elseif it > MAXITER & flag == 1
	infodat(1) = -it;
elseif it > MAXITER & flag == 0;
	error(sprintf('\nNo convergence in %d iterations (increasing MAXITER may help).\n\n',MAXITER));
end;

% ---------------------
% 3) Verification phase
if VERB
	fprintf('\n\n3. Checking obtained value\n\n');
end;

% 3.1 Checking obtained value
strF=[]; strARG=[];
for i=1:N
	% 3.1.1 Auxiliary values
	Ei=unstack(Evec,i);
	Ai=unstack(Avec,i); Bi=unstack(Bvec,i); Di=unstack(Dvec,i);
	istr=int2str(i);
	auxF=[',F',istr]; strF=[strF,auxF];
	auxARG=[',ARG',istr]; strARG=[strARG,auxARG];

	% 3.1.2 Condition (i): I - Di'*Ei(X)*Di/nu^2 > 0
	Si = IW - Di'*Ei*Di/(nu^2); invSi = inv(Si); mineigSi=min(eig(Si));
	if     mineigSi <= 0 & flag == 0,
		error('\nNo solution. Condition (i) (I-Di''Ei(X)Di/nu^2 > 0) failed\n');
	elseif mineigSi <= 0 & flag == 1,
		infodat(2) = 1;
	end;

	% 3.1.3 Calculating Fi and Gi
	Fi = -inv(IU + Bi'*Ei*Bi + Bi'*Ei*Di/(nu^2)*invSi*Di'*Ei*Bi)*...
	      Bi'*(IX - Ei*Di*invSi*Di'/(nu^2))*Ei*Ai;
	eval(['F',istr,'=Fi;']);

	Gi = invSi*Di'*Ei*(Ai+Bi*Fi)/(nu^2);

	% Calculating argument for operator L
	ARGi = Ai+Bi*Fi+Di*Gi/nu;
	eval(['ARG',istr,'=ARGi;']);
end;
strF(1)=[];   eval(['Fvec=stack(',strF,');']);
strARG(1)=[]; eval(['ARGvec=stack(',strARG,');']);		

% 3.1.4 Condition (ii): specrad(L) < 1
srARG = specrad(ARGvec,P);
if     srARG >= 1 & flag == 0,
	error('\nNo solution. Condition (ii) (specrad(L) < 1) failed\n');
elseif srARG >= 1 & flag == 1,
	infodat(3) = 1;
end;

% ---------------------
% 4) Conclusion
Xvec = Xnew;
if VERB
	fprintf('\n\n-------------------------------\n');
	fprintf(    'hinfsopt terminated succesfully\n');
	fprintf(    '-------------------------------\n\n');
end;

