function [Z,X,W,C] = l2sim(Avec,Bvec,Fvec,Dvec,Mvec,Jvec,P,x0,th0,n,wopt,s);

%L2SIM	Simulation of a closed loop system subject to L2 disturbances
%	for the Markovian case.
%
%	Usage:
%		[Z,X,W,C] = l2sim(Avec,Bvec,Fvec,Dvec,Mvec,Jvec,P,X0,TH0,N,Wopt,S)
%
%	Z,X contain the output and state respectively. W contains the disturbance
%	applied to the system and C is the realization of the Markov chain.
%
%	Avec,...,Jvec contain the system description, P is the transition probability
%	matrix, X0 and TH0 are the initial condition for the state and the Markov
%	chain respectively, N is the number of steps for the simulation, Wopt (dis-
%	cussed below) is the noise description and S (optional) is the seed for the
%	random number generator.
%
%	The noise is given by w(k) = (a^k)*r(k), where 0 < a < 1 and r(k) is a
%	normally distributed random variable. Properties of the noise are specified
%	by Wopt:
%		Wopt(1) = a.............if Wopt(1)=0, then no disturbances are considered
%		Wopt(2) = sigma.........variance of the normal distribution of r(k)
%		Wopt(3) = sw............(optional) seed for the disturbance
%					(does not affect the generation of the Markov
%					 chain realization, controlled by argument S)

%	Version 1.0
%	RP Marques	(13/03/97)

% Argument checking
if size(P,1)==size(P,2),
	N=size(P,1);
else
	error('Invalid Transition Probability Matrix');
end;

if th0 > N | th0 < 1
	error('Invalid initial state');
end;

% Dimensions
dimW = Dvec(4); W=zeros(dimW,n);
dimX = Avec(3); X=zeros(dimX,n+1); X(:,1)=x0;
dimZ = Mvec(3); Z=zeros(dimZ,n);

% Noise generation
a=wopt(1);
if 0<a | a<1
	if length(wopt)==3,
	   randn('seed',wopt(3));
	elseif length(wopt)==2,
	   randn('seed',sum(100*clock));
	end;
	
	r = randn(dimW,n)*wopt(2);
	aux = ones(dimW,1)*(1:n);
	W = a.^(aux).*r;	   
else
	error('Invalid value for noise specification');
end;


% Markov chain generation
if nargin==10,
	s = sum(100*clock);
elseif s==[],
	s = sum(100*clock);
end;
rand('seed',s);
C = mkvchain(P,th0,n,s);

% Argument unpacking
Gvec = cloopvec(Avec,Bvec,Fvec);
Qvec = cloopvec(Mvec,Jvec,Fvec);
for i=1:N
	istr=int2str(i);
	eval(['G',istr,'=unstack(Gvec,',istr,');']);
	eval(['F',istr,'=unstack(Fvec,',istr,');']);
	eval(['Q',istr,'=unstack(Qvec,',istr,');']);
	eval(['D',istr,'=unstack(Dvec,',istr,');']);
end;

% Simulation
for i=1:n
	thstr=int2str(C(i));
	eval(['X(:,i+1)=G',thstr,'*X(:,i)+D',thstr,'*W(:,i);']);
	eval(['Z(:,i)=Q',thstr,'*X(:,i);']);
end;

% Conclusion
X(:,n+1)=[];
