function [Xvec,J,C] = mclqrsim(Avec,Bvec,Fvec,Qvec,Lvec,Rvec,P,x0,th0,n,nsim,s);

%MCLQRSIM Monte Carlo simulation of the LQ Regulator
%
%	Usage:
%		[Xvec,J,C] = mclqrsim(Avec,Bvec,Fvec,Qvec,Lvec,Rvec,P,x0,th0,n,nsim,s);
%
%	Xvec contains the history of the system state (unpack using UNSTACK)
%	for each simulation. J is an estimation for the quadratic cost for each
%	simulation. C contains the realization for the Markov chains.
%
%	Avec,...,Rvec contain the system and cost descriptions, P is the
%	probability transition matrix, X0 and TH0 are the initial conditions
%	for the system and for the Markov chain. N is the number of steps
%	for each simulation and NSIM is the number of simulations. S (optional)
%	is the seed for the random number generator.

%	Version 1.0
%	RP Marques	(11/03/97)

% Argument checking
if nargin==11,
	s = sum(100*clock);
elseif s==[],
	s = sum(100*clock);
end;
rand('seed',s);

if size(P,1)==size(P,2),
	N=size(P,1);
else
	error('Invalid Transition Probability Matrix');
end;

if th0 > N | th0 < 1
	error('Invalid initial state');
end;

% Initialization
dimX=Avec(3); Xvec=zeros(2+nsim*2+nsim*dimX*n,1);
Xvec(1)=nan; Xvec(2)=nsim;
for i=1:nsim,
	Xvec(2*i+1:2*i+2)=[dimX;n];
end;

C=zeros(n,nsim); saux=floor(1e7*rand(nsim,1));

J=zeros(1,nsim);

% Simulation
fprintf('\n\nWorking...\n\n');
fprintf('000      50      100\n');
fprintf('+--------++--------+\n');

istar = 1;

for i=1:nsim,
	% Simulate
	[Xaux,J(i),C(:,i)] = lqrsim(Avec,Bvec,Fvec,Qvec,Lvec,Rvec,P,x0,th0,n,saux(i));

	% Put Xaux into Xvec
	Xvec(2+nsim*2+(i-1)*dimX*n+1:2+nsim*2+i*dimX*n) = Xaux(:);

	% Tell the operator about computation progress
	if floor(i*20/nsim) >= istar;
	   fprintf('*');
	   istar = istar+1;
	end;
end;

% Finalization
for i=istar:20;
	fprintf('*');
end;
fprintf('\n\n');
end;

	