function [Fvec,upbH2,Xvec,Qvec,Lvec,Yvec] = mixopt(Avec,Bvec,D,Mvec,Jvec,P,nu,options,v)
%MIXOPT	Finds an approximation for the solution of the mixed
%	H2/Hinf control problem.
%
%	Usage:
%		Fvec = mixopt(Avec,Bvec,D,Mvec,Jvec,P,NU,OPTIONS,V);
%		or
%		[Fvec,UPBH2,Xvec,Qvec,Lvec,Yvec] = mixopt(...);
%
%	where Avec,...,Jvec contain the system description (note that D must
%	be a matrix and not a set of matrices), P is the transition probability
%	matrix, NU is an upper bound for the Hinf-Norm, OPTIONS contains options
%	for the optimization algorithm (type HELP MINCX for more information) and
%	V controls output of information about the construction of the optimization
%	problem.

%	Alternatively, P may be a set of transition probability matrices (Pvec) 
%	defining the vertices of a polytope.

%	Fvec is the controller, UPBH2 is an upper bound for the H2-norm and 
%	Xvec,...,Yvec are the solutions for the convex programming problem associated
%	to the control problem.


%	Version 1.0
%	RP Marques	(05/3/97)


% Parameter checking
if ~( isnan(Avec(1)) & isnan(Bvec(1)) & isnan(Mvec(1)) & isnan(Jvec(1)) )
	error('Invalid argument(s)')
end;

% Problem dimensions
N = Avec(2);	% number of operation modes
n = Avec(3);	% state dimension
m = Bvec(4);	% input dimension

if (Bvec(2)~=N) | (Mvec(2)~=N) | (Jvec(2)~=N)
	error('Invalid number of operation modes for some argument(s)')
end;
if nargin<9, v=0; end;
if nargin<8, options=zeros(1,5); end;

% Argument unpacking
for i=1:N
	istr=int2str(i);
	eval(['A',istr,'=unstack(Avec,',istr,');']);
	eval(['B',istr,'=unstack(Bvec,',istr,');']);
	eval(['M',istr,'=unstack(Mvec,',istr,');']);
	eval(['J',istr,'=unstack(Jvec,',istr,');']);
end;

% P matrix
if isnan(P(1)),
	% P belongs to a polytope
	q = P(2);
	for i=1:N
		istr=int2str(i);
		eval(['P',istr,'=unstack(P,',istr,');']);
	end;
else
	% P is a single matrix
	q  = 1;
	P1 = P;
end;

% Other parameters
mu = nu^2;
In = eye(n);

for i=1:N
	istr=int2str(i);
	for j=1:N
		jstr=int2str(j);
		for t=1:q
			tstr=int2str(t);
			% Gijt=sqrt(Pt(i,j))*In
			eval(['G',istr,jstr,tstr,'=sqrt(P',tstr,'(i,j))*In;']);
		end;
	end;
end;

% Problem formulation

%%% 1. initialization  
setlmis([]);
if v, fprintf('\n\nProblem construction:\n\n\n'); end;

%%% 2. Variable declaration  
if v, fprintf('1) Variable declaration\n\n'); end;     
for i=1:N
	str = ['X',int2str(i),'=lmivar(1,[',int2str(n),' 1]);'];
	eval(str); if v, disp(str); end;
	str = ['Q',int2str(i),'=lmivar(1,[',int2str(n),' 1]);'];
	eval(str); if v, disp(str); end;
	str = ['L',int2str(i),'=lmivar(1,[',int2str(n),' 1]);'];
	eval(str); if v, disp(str); end;
	str = ['Y',int2str(i),'=lmivar(2,[',int2str(m),' ',int2str(n),']);'];
	eval(str); if v, disp(str); end;
end;

%%% 3. Restriction declaration
if v, fprintf('\n2) Inequalities\n\n'); end;

eqind = 1; eqstr = int2str(eqind);
for i=1:N
	istr=int2str(i);

	% Inequality #1
	str = ['lmiterm([-',eqstr,' 1 1  Q',istr,'],1,1);'];
	eval(str); if v, disp(str); end;

	str = ['lmiterm([-',eqstr,' 1 2  Q',istr,'],1,A',istr,''');'];
	eval(str); if v, disp(str); end;
	str = ['lmiterm([-',eqstr,' 1 2 -Y',istr,'],1,B',istr,''');'];
	eval(str); if v, disp(str); end;

	str = ['lmiterm([-',eqstr,' 1 3  Q',istr,'],1,M',istr,''');'];
	eval(str); if v, disp(str); end;

	str = ['lmiterm([-',eqstr,' 1 4 -Y',istr,'],1,J',istr,''');'];
	eval(str); if v, disp(str); end;

	str = ['lmiterm([-',eqstr,' 1 5 0],D);'];
	eval(str); if v, disp(str); end;

	str = ['lmiterm([-',eqstr,' 2 2  L',istr,'],1,1);'];
	eval(str); if v, disp(str); end;

	str = ['lmiterm([-',eqstr,' 3 3 0],1);'];
	eval(str); if v, disp(str); end;

	str = ['lmiterm([-',eqstr,' 4 4 0],1);'];
	eval(str); if v, disp(str); end;

	str = ['lmiterm([-',eqstr,' 5 5 0],mu*In);'];
	eval(str); if v, disp(str); end;

	eqind=eqind+1; eqstr=int2str(eqind);

	% Inequality #2
	for t=1:q
	   tstr=int2str(t);

	   str = ['lmiterm([-',eqstr,' 1 1 L',istr,'],1,1);'];
	   eval(str); if v, disp(str); end;

	   for j=1:N
	      jstr=int2str(j); j1str=int2str(j+1);
	      str = ['lmiterm([-',eqstr,' 1 ',j1str,' L',istr,'],1,G',istr,jstr,tstr,');'];
	      eval(str); if v, disp(str); end;

	      str = ['lmiterm([-',eqstr,' ',j1str,' ',j1str,' Q',jstr,'],1,1);'];
	      eval(str); if v, disp(str); end;
	   end;

	   eqind=eqind+1; eqstr=int2str(eqind);

	end;

	% Inequality #3
	str = ['lmiterm([-',eqstr,' 1 1 X',istr,'],1,1);'];
	eval(str); if v, disp(str); end;

	str = ['lmiterm([-',eqstr,' 1 2 0],1);'];
	eval(str); if v, disp(str); end;

	str = ['lmiterm([-',eqstr,' 2 2 Q',istr,'],1,1);'];
	eval(str); if v, disp(str); end;

	eqind=eqind+1; eqstr=int2str(eqind);

	% Inequalities #4,#5,#6
	str = ['lmiterm([-',eqstr,' 1 1 X',istr,'],1,1);'];
	eval(str); if v, disp(str); end;

	eqind=eqind+1; eqstr=int2str(eqind);

	str = ['lmiterm([-',eqstr,' 1 1 Q',istr,'],1,1);'];
	eval(str); if v, disp(str); end;

	eqind=eqind+1; eqstr=int2str(eqind);

	str = ['lmiterm([-',eqstr,' 1 1 L',istr,'],1,1);'];
	eval(str); if v, disp(str); end;

	eqind=eqind+1; eqstr=int2str(eqind);


end;

%%% 4. Conclusion
sistema=getlmis;

% Objective function declaration
% (see page 8-30)

nsis=decnbr(sistema); c=zeros(nsis,1);

if v, fprintf('\n3) Calculating Objective Function.\n\n'); end;
str1=[]; str2=[]; str3=[];
for i=1:N
	istr=int2str(i);
	aux1=[',X',istr,',Q',istr,',L',istr,',Y',istr];
	aux2=[',X',istr,'j,Q',istr,'j,L',istr,'j,Y',istr,'j'];
	aux3=['+X',istr,'j'];
	str1=[str1,aux1]; str2=[str2,aux2]; str3=[str3,aux3];
end;
str1(1)=[]; str2(1)=[]; str3(1)=[];

for j=1:nsis
	eval(['[',str2,']=defcx(sistema,j,',str1,');']);
	eval(['c(j)=trace(D''*(',str3,')*D);']);
end;

% Optimization
[copt,xopt]=mincx(sistema,c,options,[],[]);

% Output
str1=[]; str2=[]; str3=[]; str4=[]; str5=[];
for i=1:N
	istr=int2str(i);
	aux1=[',X',istr,'opt'];
	aux2=[',Q',istr,'opt'];
	aux3=[',L',istr,'opt'];
	aux4=[',Y',istr,'opt'];
	aux5=[',F',istr];
	aux6=['+X',istr,'opt'];

	eval(['X',istr,'opt=dec2mat(sistema,xopt,X',istr,');']);
	eval(['Q',istr,'opt=dec2mat(sistema,xopt,Q',istr,');']);
	eval(['L',istr,'opt=dec2mat(sistema,xopt,L',istr,');']);
	eval(['Y',istr,'opt=dec2mat(sistema,xopt,Y',istr,');']);
	eval(['F',istr,'=Y',istr,'opt*inv(Q',istr,'opt);']);

	str1=[str1,aux1]; str2=[str2,aux2]; str3=[str3,aux3];
	str4=[str4,aux4]; str5=[str5,aux5]; str6=[str6,aux6];
end;

str1(1)=[]; str2(1)=[]; str3(1)=[]; str4(1)=[]; str5(1)=[]; str6(1)=[];

eval(['Xvec=stack(',str1,');']);
eval(['Qvec=stack(',str2,');']);
eval(['Lvec=stack(',str3,');']);
eval(['Yvec=stack(',str4,');']);
eval(['Fvec=stack(',str5,');']);
upbH2 = copt;





